package xyz.lychee.lagfixer.managers;

import com.google.common.collect.Lists;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.Language;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.resolver.Placeholder;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.resolver.TagResolver;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.map.MapHandler;
import xyz.lychee.lagfixer.menu.MainMenu;
import xyz.lychee.lagfixer.menu.ModuleMenu;
import xyz.lychee.lagfixer.menu.ModulesMenu;
import xyz.lychee.lagfixer.modules.WorldCleanerModule;
import xyz.lychee.lagfixer.objects.AbstractManager;
import xyz.lychee.lagfixer.objects.AbstractModule;
import xyz.lychee.lagfixer.objects.AbstractMonitor;
import xyz.lychee.lagfixer.utils.MessageUtils;
import xyz.lychee.lagfixer.utils.TimingUtil;

public class CommandManager extends AbstractManager implements Listener, TabExecutor {
   private static CommandManager instance;
   private final HashMap<UUID, ConfigChange> playerchange = new HashMap();
   private final MainMenu mainMenu;
   private final ModulesMenu modulesMenu;
   private final MapHandler mapHandler;
   private String permission;
   private boolean benchmark = false;

   public CommandManager(LagFixer plugin) {
      super(plugin);
      instance = this;
      this.mainMenu = new MainMenu(plugin, 27, MessageUtils.fixColors((CommandSender)null, "&8[&e&l⚡&8] &fMenu! &8| &eLagFixer"));
      this.modulesMenu = new ModulesMenu(plugin, 45, MessageUtils.fixColors((CommandSender)null, "&8[&e&l⚡&8] &fModules! &8| &eLagFixer"));
      this.mapHandler = new MapHandler(plugin);
   }

   @EventHandler
   public void onChat(AsyncPlayerChatEvent e) {
      if (this.playerchange.containsKey(e.getPlayer().getUniqueId())) {
         e.setCancelled(true);
         ConfigChange change = (ConfigChange)this.playerchange.get(e.getPlayer().getUniqueId());
         this.playerchange.remove(e.getPlayer().getUniqueId());
         if (e.getMessage().equalsIgnoreCase("-cancel") || e.getMessage().equalsIgnoreCase("cancel")) {
            SupportManager.getInstance().getFork().runSync(e.getPlayer().getLocation(), () -> {
               e.getPlayer().openInventory(change.getModule().getMenu().getInv());
            });
            MessageUtils.sendMessage(true, e.getPlayer(), "Config editor has been successfully canceled!");
            return;
         }

         ConfigurationSection section = change.getModule().getSection();
         String key = change.getKey();
         String msg = e.getMessage();

         try {
            if (section.isString(key)) {
               section.set(key, msg);
            } else if (section.isInt(key)) {
               int i = Integer.parseInt(msg);
               section.set(key, i);
            } else if (section.isDouble(key)) {
               double d = Double.parseDouble(msg);
               section.set(key, d);
            } else if (section.isLong(key)) {
               long l = Long.parseLong(msg);
               section.set(key, l);
            } else if (section.isBoolean(key)) {
               boolean b = Boolean.parseBoolean(msg);
               section.set(key, b);
            } else if (section.isList(key)) {
               List<String> list = section.getStringList(key);
               if (list.contains(msg)) {
                  list.remove(msg);
               } else {
                  list.add(msg);
               }

               section.set(key, list);
            }

            this.getPlugin().saveConfig();
            change.getModule().loadAllConfig();
            MessageUtils.sendMessage(true, e.getPlayer(), "&fConfiguration successfully saved!\n \n &8{*} &fOld value:\n &r &r &e" + change.getValue() + "\n \n &8{*} &fNew value:\n &r &r &e" + section.get(key) + "\n ");
            SupportManager.getInstance().getFork().runSync(e.getPlayer().getLocation(), () -> {
               ModuleMenu menu = change.getModule().getMenu();
               menu.update();
               e.getPlayer().openInventory(menu.getInv());
            });
         } catch (Exception var8) {
            Exception ex = var8;
            MessageUtils.sendMessage(true, e.getPlayer(), "&fAn error occurred while saving the configuration, make sure that the given value is correct!");
            this.getPlugin().printError(ex);
         }
      }

   }

   @EventHandler
   public void onQuit(PlayerQuitEvent e) {
      this.playerchange.remove(e.getPlayer().getUniqueId());
   }

   public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
      if (!sender.hasPermission(this.permission)) {
         Component text = Language.getMainValue("no_access", true, new TagResolver.Single[]{Placeholder.unparsed("permission", this.permission)});
         if (text == null) {
            return false;
         } else {
            LagFixer.getInstance().getAudiences().sender(sender).sendMessage(text);
            return false;
         }
      } else if (args.length > 0) {
         WorldCleanerModule module = (WorldCleanerModule)ModuleManager.getInstance().get(WorldCleanerModule.class);
         List projectiles;
         switch (String.join(" ", args).toLowerCase()) {
            case "menu":
               if (sender instanceof Player) {
                  ((Player)sender).openInventory(this.mainMenu.getInv());
               } else {
                  Component text = Language.getMainValue("player_only", true, new TagResolver.Single[0]);
                  if (text != null) {
                     LagFixer.getInstance().getAudiences().sender(sender).sendMessage(text);
                  }
               }

               return false;
            case "benchmark":
               if (this.benchmark) {
                  return MessageUtils.sendMessage(true, sender, "&7Benchmark is running, wait for results!");
               } else {
                  this.benchmark = true;
                  MessageUtils.sendMessage(true, sender, "&7Loading async benchmark...");
                  if (SupportManager.getInstance().getMonitor().getMspt() > 10.0) {
                     return MessageUtils.sendMessage(false, sender, "&cServer MSPT is too high, the result may be incorrect&4!");
                  }

                  ExecutorService executor = Executors.newSingleThreadExecutor();
                  executor.execute(() -> {
                     long start = System.currentTimeMillis() + 1500L;

                     int score;
                     for(score = 0; start > System.currentTimeMillis(); ++score) {
                     }

                     MessageUtils.sendMessage(false, sender, "&8{*} &7Benchmark score is &e" + score + " &7points.\nWe are working on an option to compare the result with other servers.");
                     this.benchmark = false;
                  });
                  executor.shutdown();
                  return false;
               }
            case "reload":
               TimingUtil t = TimingUtil.startNew();
               LagFixer main = this.getPlugin();
               Language.loadLangYaml(main);
               main.reloadConfig();
               ModuleManager.getInstance().getModules().forEach((clazz, m) -> {
                  try {
                     m.loadAllConfig();
                     main.getLogger().info("&fConfiguration for &a" + m.getName() + " &fsuccessfully reloaded!");
                  } catch (Exception var4) {
                     Exception ex = var4;
                     main.printError(ex);
                     main.getLogger().info("&fError reloading configuration for &c" + m.getName() + "&f!");
                  }

               });
               MessageUtils.sendMessage(true, sender, "&7Reloaded some configurations in &f" + t.stop().getExecutingTime() + "&7ms.\n \n &7Working methods to apply all changes:\n  &8{*} &7Server restart (&frecommended&7)\n  &8{*} &7All plugins reload, command: &f/reload confirm\n  &8{*} &7Plugman reload, command: &f/plugman reload LagFixer");
               return false;
            case "tps":
            case "mspt":
            case "monitor":
               AbstractMonitor monitor = SupportManager.getInstance().getMonitor();
               return MessageUtils.sendMessage(true, sender, "&7Command result: \n &8{*} &fTps: &e" + monitor.getTps() + "\n &8{*} &fMspt: &e" + monitor.getMspt() + "\n &8{*} &fMemory: &e" + monitor.getRamUsed() + "&8/&e" + monitor.getRamTotal() + "&8/&e" + monitor.getRamMax() + " MB\n &8{*} &fCpu process: &e" + monitor.getCpuProcess() + "%\n &8{*} &fCpu system: &e" + monitor.getCpuSystem() + "%");
            case "free":
               Runtime runtime = Runtime.getRuntime();
               long oram = runtime.totalMemory() - runtime.freeMemory();
               runtime.gc();
               long nram = runtime.totalMemory() - runtime.freeMemory();
               return MessageUtils.sendMessage(true, sender, "&7Command result:\n&8{*} &fGarbage collector cleared &e" + (oram - nram) + "bytes &fram!");
            case "ping":
               return MessageUtils.sendMessage(true, sender, "&7Command result:\n&8{*} &fAverage players ping: &e" + SupportManager.getInstance().getNms().averagePing());
            case "clear items":
               if (module != null && module.isLoaded()) {
                  projectiles = (List)module.clearItems().collect(Collectors.toList());
                  SupportManager.getInstance().getItemMap().values().forEach((set) -> {
                     Objects.requireNonNull(set);
                     projectiles.forEach(set::remove);
                  });
                  projectiles.forEach(Entity::remove);
                  MessageUtils.sendMessage(true, sender, "&7Successfully removed &e" + projectiles.size() + " &7items.");
               } else {
                  MessageUtils.sendMessage(true, sender, "&7WorldCleaner module is disabled, so LagFixer cannot remove the items from the indicated rules!");
               }

               return false;
            case "clear creatures":
               if (module != null && module.isLoaded()) {
                  projectiles = (List)module.clearCreatures().collect(Collectors.toList());
                  SupportManager.getInstance().getEntityMap().values().forEach((set) -> {
                     Objects.requireNonNull(set);
                     projectiles.forEach(set::remove);
                  });
                  projectiles.forEach(Entity::remove);
                  MessageUtils.sendMessage(true, sender, "&7Successfully removed &e" + projectiles.size() + " &7creatures.");
               } else {
                  MessageUtils.sendMessage(true, sender, "&7WorldCleaner module is disabled, so LagFixer cannot remove the mobs from the indicated rules!");
               }

               return false;
            case "clear projectiles":
               if (module != null && module.isLoaded()) {
                  projectiles = (List)module.clearProjectiles().collect(Collectors.toList());
                  SupportManager.getInstance().getProjectileMap().values().forEach((set) -> {
                     Objects.requireNonNull(set);
                     projectiles.forEach(set::remove);
                  });
                  projectiles.forEach(Entity::remove);
                  MessageUtils.sendMessage(true, sender, "&7Successfully removed &e" + projectiles.size() + " &7projectiles.");
               } else {
                  MessageUtils.sendMessage(true, sender, "&7WorldCleaner module is disabled, so LagFixer cannot remove the mobs from the indicated rules!");
               }
            case "map":
               if (sender instanceof Player) {
                  this.mapHandler.giveMap((Player)sender);
               } else {
                  Component text = Language.getMainValue("player_only", true, new TagResolver.Single[0]);
                  if (text != null) {
                     LagFixer.getInstance().getAudiences().sender(sender).sendMessage(text);
                  }
               }

               return false;
            default:
               return false;
         }
      } else {
         return MessageUtils.sendMessage(true, sender, "&8{*} &f/lagfixer &emenu &8- &7open lagfixer menu, and edit configuration\n&8{*} &f/lagfixer &ereload &8- &7reload all plugin configuration\n&8{*} &f/lagfixer &etps &8- &7check server load statistics\n&8{*} &f/lagfixer &ebenchmark &8- &7run benchmark and compare it with other servers\n&8{*} &f/lagfixer &eping &8- &7calculate average players ping\n&8{*} &f/lagfixer &efree &8- &7clear free ram from memory\n&8{*} &f/lagfixer &eclear items &8- &7remove items using rules in WorldCleaner\n&8{*} &f/lagfixer &eclear creatures &8- &7remove mobs using rules in WorldCleaner\n&8{*} &f/lagfixer &eclear projectiles &8- &7remove projectiles using rules in WorldCleaner\n&8{*} &f/lagfixer &emap &8- &7server load map monitor with mspt chart");
      }
   }

   public @Nullable List<String> onTabComplete(@NotNull CommandSender commandSender, @NotNull Command command, @NotNull String s, @NotNull String[] args) {
      if (args.length == 1) {
         List<String> completions = Lists.newArrayList(new String[]{"menu", "benchmark", "reload", "tps", "free", "ping", "clear items", "clear creatures", "map"});
         if (!args[0].isEmpty()) {
            completions.removeIf((str) -> {
               return !str.startsWith(args[0]);
            });
         }

         Collections.sort(completions);
         return completions;
      } else {
         return (List)Bukkit.getOnlinePlayers().stream().map(OfflinePlayer::getName).collect(Collectors.toList());
      }
   }

   public void load() {
      this.modulesMenu.update();
      this.mapHandler.load(this.getPlugin().getConfig().getInt("main.map_interval"));
      this.permission = this.getPlugin().getConfig().getString("main.command.permission");
      SupportManager.getInstance().getFork().registerCommand(this.getPlugin(), "lagfixer", this.getPlugin().getConfig().getStringList("main.command.aliases"), this);
      this.getPlugin().getServer().getPluginManager().registerEvents(this, this.getPlugin());
   }

   public void disable() {
      HandlerList.unregisterAll(this);
      this.mainMenu.unload();
      this.modulesMenu.unload();
      this.mapHandler.unload();
   }

   public boolean isEnabled() {
      return true;
   }

   public HashMap<UUID, ConfigChange> getPlayerchange() {
      return this.playerchange;
   }

   public MainMenu getMainMenu() {
      return this.mainMenu;
   }

   public ModulesMenu getModulesMenu() {
      return this.modulesMenu;
   }

   public MapHandler getMapHandler() {
      return this.mapHandler;
   }

   public String getPermission() {
      return this.permission;
   }

   public boolean isBenchmark() {
      return this.benchmark;
   }

   public static CommandManager getInstance() {
      return instance;
   }

   public static class ConfigChange {
      private final AbstractModule module;
      private final String key;
      private final Object value;

      public ConfigChange(AbstractModule module, String key, Object value) {
         this.module = module;
         this.key = key;
         this.value = value;
      }

      public AbstractModule getModule() {
         return this.module;
      }

      public String getKey() {
         return this.key;
      }

      public Object getValue() {
         return this.value;
      }

      public boolean equals(Object o) {
         if (o == this) {
            return true;
         } else if (!(o instanceof ConfigChange)) {
            return false;
         } else {
            ConfigChange other = (ConfigChange)o;
            if (!other.canEqual(this)) {
               return false;
            } else {
               label47: {
                  Object this$module = this.getModule();
                  Object other$module = other.getModule();
                  if (this$module == null) {
                     if (other$module == null) {
                        break label47;
                     }
                  } else if (this$module.equals(other$module)) {
                     break label47;
                  }

                  return false;
               }

               Object this$key = this.getKey();
               Object other$key = other.getKey();
               if (this$key == null) {
                  if (other$key != null) {
                     return false;
                  }
               } else if (!this$key.equals(other$key)) {
                  return false;
               }

               Object this$value = this.getValue();
               Object other$value = other.getValue();
               if (this$value == null) {
                  if (other$value != null) {
                     return false;
                  }
               } else if (!this$value.equals(other$value)) {
                  return false;
               }

               return true;
            }
         }
      }

      protected boolean canEqual(Object other) {
         return other instanceof ConfigChange;
      }

      public int hashCode() {
         int PRIME = true;
         int result = 1;
         Object $module = this.getModule();
         result = result * 59 + ($module == null ? 43 : $module.hashCode());
         Object $key = this.getKey();
         result = result * 59 + ($key == null ? 43 : $key.hashCode());
         Object $value = this.getValue();
         result = result * 59 + ($value == null ? 43 : $value.hashCode());
         return result;
      }

      public String toString() {
         return "CommandManager.ConfigChange(module=" + this.getModule() + ", key=" + this.getKey() + ", value=" + this.getValue() + ")";
      }
   }
}
